from .publishevent import *
from .consumeevent import *

__all__ = [
    "PublishEventRequest", "PublishEventRequestBody", "PublishEventResponse",
    "PublishEventResponseBody", "ConsumeEventRequest", "ConsumeEventResponse",
    "ConsumeEventResponseBody"
]



from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from glassflow import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Error(Exception):
    """Bad request error response

    Attributes:
        message: A message describing the error

    """
    message: str = dataclasses.field(metadata={
        'dataclasses_json': {
            'letter_case': utils.get_field_name('message')
        }
    })

    def __str__(self) -> str:
        return utils.marshal_json(self, type(self))



"""Dataclasses for the consume event operation
"""

from __future__ import annotations
import dataclasses
import requests as requests_http
from typing import Optional
from dataclasses_json import config, dataclass_json


@dataclasses.dataclass
class ConsumeEventRequest:
    """Request to consume an event from a pipeline topic

    Attributes:
        pipeline_id: The id of the pipeline
        space_id: The id of the space
        organization_id: The id of the organization
        x_pipeline_access_token: The access token of the pipeline

    """
    pipeline_id: str = dataclasses.field(
        metadata={
            'path_param': {
                'field_name': 'pipeline_id',
                'style': 'simple',
                'explode': False
            }
        })
    space_id: str = dataclasses.field(
        metadata={
            'query_param': {
                'field_name': 'space_id',
                'style': 'form',
                'explode': True
            }
        })
    organization_id: Optional[str] = dataclasses.field(
        default=None,
        metadata={
            'query_param': {
                'field_name': 'organization_id',
                'style': 'form',
                'explode': True
            }
        })
    x_pipeline_access_token: str = dataclasses.field(
        default=None,
        metadata={
            'header': {
                'field_name': 'X-PIPELINE-ACCESS-TOKEN',
                'style': 'simple',
                'explode': False
            }
        })


@dataclass_json
@dataclasses.dataclass
class ConsumeEventResponseBody:
    """Event response body after transformation

    Attributes:
        req_id: The request id
        receive_time: The time when the event was received
        event: The event received

    """
    req_id: str = dataclasses.field()
    receive_time: str = dataclasses.field()
    event: dict = dataclasses.field(metadata=config(field_name="response"))


@dataclasses.dataclass
class ConsumeEventResponse:
    """Response to consume an event from a pipeline topic

    Attributes:
        content_type: HTTP response content type for this operation
        status_code: HTTP response status code for this operation
        raw_response: Raw HTTP response; suitable for custom response parsing
        body: the response body from the

    """
    content_type: str = dataclasses.field()
    status_code: int = dataclasses.field()
    raw_response: requests_http.Response = dataclasses.field()
    body: Optional[ConsumeEventResponseBody] = dataclasses.field(default=None)



from dataclasses import dataclass
import requests
from typing import Tuple


@dataclass
class GlassFlowConfig:
    """Configuration object for GlassFlowClient

    Attributes:
        client: requests.Session object to interact with the GlassFlow API
        server_url: The base URL of the GlassFlow API
        sdk_version: The version of the GlassFlow Python SDK
        user_agent: The user agent to be used in the requests

    """
    client: requests.Session
    server_url: str = 'http://api.glassflow.xyz/v1'
    sdk_version: str = '0.0.1'
    user_agent: str = 'glassflow-python-sdk'



import requests as requests_http


class ClientError(Exception):
    """Represents an error returned by the API.

    Attributes:
        message: A message describing the error
        status_code: The status code of the response
        body: The response body
        raw_response: The raw response object

    """
    message: str
    status_code: int
    body: str
    raw_response: requests_http.Response

    def __init__(self, message: str, status_code: int, body: str,
                 raw_response: requests_http.Response):
        """Create a new ClientError object

        Args:
            message: A message describing the error
            status_code: The status code of the response
            body: The response body
            raw_response: The raw response object
        """
        self.message = message
        self.status_code = status_code
        self.body = body
        self.raw_response = raw_response

    def __str__(self):
        """Return a string representation of the error

        Returns:
            str: The string representation of the error

        """
        body = ''
        if len(self.body) > 0:
            body = f'\n{self.body}'

        return f'{self.message}: Status {self.status_code}{body}'



from .models import operations, errors
from typing import Dict, Optional
import glassflow.utils as utils


class PipelineClient():
    """Client object to publish and consume events from the given pipeline.

    Attributes:
        glassflow_client: GlassFlowClient object to interact with GlassFlow API
        space_id: The space id where the pipeline is located
        pipeline_id: The pipeline id to interact with
        organization_id: Organization ID of the user. If not provided, the default organization will be used
    """

    def __init__(self, glassflow_client, space_id: str,
                 pipeline_id: str) -> None:
        """Create a new PipelineClient object to interact with a specific pipeline

        Args:
            glassflow_client: GlassFlowClient object to interact with GlassFlow API
            space_id: The space id where the pipeline is located
            pipeline_id: The pipeline id to interact with
        """
        self.glassflow_client = glassflow_client
        self.space_id = space_id
        self.pipeline_id = pipeline_id
        self.organization_id = self.glassflow_client.organization_id

    def publish(self, request_body: dict,
                pipeline_access_token: str) -> operations.PublishEventResponse:
        """Push a new message into the pipeline

        Args:
            request_body: The message to be published into the pipeline
            pipeline_access_token: The access token to access the pipeline

        Returns:
            PublishEventResponse: Response object containing the status code and the raw response

        Raises:
            ClientError: If an error occurred while publishing the event
        """
        request = operations.PublishEventRequest(
            organization_id=self.organization_id,
            space_id=self.space_id,
            pipeline_id=self.pipeline_id,
            x_pipeline_access_token=pipeline_access_token,
            request_body=request_body,
        )

        base_url = self.glassflow_client.glassflow_config.server_url

        url = utils.generate_url(
            operations.PublishEventRequest, base_url,
            '/pipelines/{pipeline_id}/topics/input/events', request)
        headers = utils.get_headers(request)

        req_content_type, data, form = utils.serialize_request_body(
            request, operations.PublishEventRequest, "request_body", False,
            True, 'json')
        if req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type

        query_params = utils.get_query_params(operations.PublishEventRequest,
                                              request)

        headers['Accept'] = 'application/json'
        headers[
            'user-agent'] = self.glassflow_client.glassflow_config.user_agent

        client = self.glassflow_client.glassflow_config.client

        http_res = client.request('POST',
                                  url,
                                  params=query_params,
                                  data=data,
                                  files=form,
                                  headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.PublishEventResponse(status_code=http_res.status_code,
                                              content_type=content_type,
                                              raw_response=http_res)

        if http_res.status_code == 200:
            pass
        elif http_res.status_code in [400, 500]:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.Error)
                out.raw_response = http_res
                raise out
            else:
                raise errors.ClientError(
                    f'unknown content-type received: {content_type}',
                    http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401 or http_res.status_code == 404 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.ClientError('API error occurred',
                                     http_res.status_code, http_res.text,
                                     http_res)

        return res

    def consume(self,
                pipeline_access_token: str) -> operations.ConsumeEventResponse:
        """Consume the last message from the pipeline

        Args:
            pipeline_access_token: The access token to access the pipeline

        Returns:
            ConsumeEventResponse: Response object containing the status code and the raw response

        Raises:
            ClientError: If an error occurred while consuming the event

        """
        request = operations.ConsumeEventRequest(
            space_id=self.space_id,
            pipeline_id=self.pipeline_id,
            organization_id=self.organization_id,
            x_pipeline_access_token=pipeline_access_token,
        )

        base_url = self.glassflow_client.glassflow_config.server_url

        url = utils.generate_url(
            operations.ConsumeEventRequest, base_url,
            '/pipelines/{pipeline_id}/topics/output/events/consume', request)
        headers = utils.get_headers(request)
        query_params = utils.get_query_params(operations.ConsumeEventRequest,
                                              request)
        headers['Accept'] = 'application/json'
        headers[
            'user-agent'] = self.glassflow_client.glassflow_config.user_agent
        client = self.glassflow_client.glassflow_config.client
        http_res = client.request('POST',
                                  url,
                                  params=query_params,
                                  headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.ConsumeEventResponse(status_code=http_res.status_code,
                                              content_type=content_type,
                                              raw_response=http_res)

        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                body = utils.unmarshal_json(
                    http_res.text,
                    Optional[operations.ConsumeEventResponseBody])
                res.body = body
            else:
                raise errors.ClientError(
                    f'unknown content-type received: {content_type}',
                    http_res.status_code, http_res.text, http_res)
        elif http_res.status_code in [400, 500]:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.Error)
                out.raw_response = http_res
                raise out
            else:
                raise errors.ClientError(
                    f'unknown content-type received: {content_type}',
                    http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401 or http_res.status_code == 404 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.ClientError('API error occurred',
                                     http_res.status_code, http_res.text,
                                     http_res)

        return res



"""Dataclasses for publish event operation
"""
from __future__ import annotations
import dataclasses
import requests as requests_http
from typing import Optional


@dataclasses.dataclass
class PublishEventRequestBody:
    pass


@dataclasses.dataclass
class PublishEventRequest:
    """Request to publish an event to a pipeline topic

    Attributes:
        pipeline_id: The id of the pipeline
        space_id: The id of the space
        organization_id: The id of the organization
        x_pipeline_access_token: The access token of the pipeline
        request_body: The request body / event that should be published to the pipeline
    """
    pipeline_id: str = dataclasses.field(
        metadata={
            'path_param': {
                'field_name': 'pipeline_id',
                'style': 'simple',
                'explode': False
            }
        })
    space_id: str = dataclasses.field(
        metadata={
            'query_param': {
                'field_name': 'space_id',
                'style': 'form',
                'explode': True
            }
        })
    organization_id: Optional[str] = dataclasses.field(
        default=None,
        metadata={
            'query_param': {
                'field_name': 'organization_id',
                'style': 'form',
                'explode': True
            }
        })
    x_pipeline_access_token: str = dataclasses.field(
        default=None,
        metadata={
            'header': {
                'field_name': 'X-PIPELINE-ACCESS-TOKEN',
                'style': 'simple',
                'explode': False
            }
        })
    request_body: dict = dataclasses.field(
        default=None, metadata={'request': {
            'media_type': 'application/json'
        }})


@dataclasses.dataclass
class PublishEventResponseBody:
    """Message pushed to the pipeline"""


@dataclasses.dataclass
class PublishEventResponse:
    """Response object for publish event operation

    Attributes:
        content_type: HTTP response content type for this operation
        status_code: HTTP response status code for this operation
        raw_response: Raw HTTP response; suitable for custom response parsing
        object: Response to the publish operation

    """
    content_type: str = dataclasses.field()
    status_code: int = dataclasses.field()
    raw_response: requests_http.Response = dataclasses.field()
    object: Optional[PublishEventResponseBody] = dataclasses.field(
        default=None)



import setuptools

try:
    with open("README.md", "r") as fh:
        long_description = fh.read()
except FileNotFoundError:
    long_description = ""

setuptools.setup(
    name="glassflow",
    version="0.1.2",
    author="glassflow",
    description="GlassFlow Python Client SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(where="src"),
    install_requires=[
        "urllib3==1.26.15", "certifi>=2023.7.22", "charset-normalizer>=3.2.0",
        "dataclasses-json>=0.6.4", "idna>=3.4", "jsonpath-python>=1.0.6 ",
        "marshmallow>=3.19.0", "mypy-extensions>=1.0.0", "packaging>=23.1",
        "python-dateutil>=2.8.2", "requests>=2.31.0", "six>=1.16.0",
        "typing-inspect>=0.9.0", "typing_extensions>=4.7.1"
    ],
    extras_require={"dev": ["pylint==2.16.2"]},
    package_dir={'': 'src'},
    python_requires='>=3.8',
    package_data={"glassflow": ["py.typed"]},
)



"""GlassFlow Python Client to interact with GlassFlow API
"""

from .pipelines import PipelineClient
from .config import GlassFlowConfig
import requests as requests_http


class GlassFlowClient:
    """GlassFlow Client to interact with GlassFlow API and manage pipelines and other resources

    Attributes:
        rclient: requests.Session object to make HTTP requests to GlassFlow API
        glassflow_config: GlassFlowConfig object to store configuration
        organization_id: Organization ID of the user. If not provided, the default organization will be used

    """
    glassflow_config: GlassFlowConfig

    def __init__(self, organization_id: str = None) -> None:
        """Create a new GlassFlowClient object

        Args:
            organization_id: Organization ID of the user. If not provided, the default organization will be used
        """
        rclient = requests_http.Session()
        self.glassflow_config = GlassFlowConfig(rclient)
        self.organization_id = organization_id

    def pipeline_client(self, space_id: str,
                        pipeline_id: str) -> PipelineClient:
        """Create a new PipelineClient object to interact with a specific pipeline

        Args:
            space_id: The space id where the pipeline is located
            pipeline_id: The pipeline id to interact with

        Returns:
            PipelineClient: Client object to publish and consume events from the given pipeline.
        """
        return PipelineClient(glassflow_client=self,
                              space_id=space_id,
                              pipeline_id=pipeline_id)
